"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SignatureInput = void 0;

var _preact = require("preact");

var _button = require("./button");

var _icon = require("./icon");

var _links = require("./links");

const height = 300;

class SignatureInput extends _preact.Component {
  constructor(props) {
    super(props);
    this.mouseMove = this.mouseMove.bind(this);
    this.mouseEnd = this.mouseEnd.bind(this);
    this.mouseDown = this.mouseDown.bind(this);
    this.mouseUp = this.mouseUp.bind(this);
    this.mouseLeave = this.mouseLeave.bind(this);
    this.touchStart = this.touchStart.bind(this);
    this.touchMove = this.touchMove.bind(this);
    this.touchEnd = this.touchEnd.bind(this);
    this.clear = this.clear.bind(this);
    this.cancel = this.cancel.bind(this);
    this.calculateCanvasInfo = this.calculateCanvasInfo.bind(this);
    this.renderCanvas = this.renderCanvas.bind(this);
    this.state = {
      hasPath: false
    };
  }

  reset() {
    this.path = '';
    this.break = true;
    this.setState({
      hasPath: false
    });
  }

  componentDidMount() {
    this.reset();
    this.canvas = this.base.querySelector('canvas');
    this.ctx = this.canvas.getContext('2d');
    window.addEventListener('resize', this.calculateCanvasInfo, {
      passive: true
    });
    window.addEventListener('scroll', this.calculateCanvasInfo, {
      passive: true
    });
    document.addEventListener('mouseup', this.mouseUp);
    document.addEventListener('touchmove', this.touchMove);
    document.addEventListener('touchend', this.touchEnd);
    document.addEventListener('touchcancel', this.touchEnd);
    this.canvas.addEventListener('touchstart', this.touchStart);
    this.calculateCanvasInfo();
    this.renderCanvas();
  }

  calculateCanvasInfo() {
    const {
      top,
      left,
      width,
      height
    } = this.canvas.getBoundingClientRect();
    this.canvasInfo = {
      left,
      top,
      height,
      width
    };
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.calculateCanvasInfo, {
      passive: true
    });
    window.removeEventListener('scroll', this.calculateCanvasInfo, {
      passive: true
    });
    document.removeEventListener('mouseup', this.mouseUp);
    document.removeEventListener('touchmove', this.touchMove);
    document.removeEventListener('touchend', this.touchEnd);
    document.removeEventListener('touchcancel', this.touchEnd);
    this.canvas && this.canvas.removeEventListener('touchstart', this.touchStart);
  }

  adjustCoord(clientX, clientY) {
    const {
      left,
      top,
      height,
      width
    } = this.canvasInfo;
    const x = clientX - left;
    const y = clientY - top;

    if (x < 0 || x > width) {
      return null;
    }

    if (y < 0 || y > height) {
      return null;
    }

    return {
      x,
      y
    };
  }

  appendCoord(x, y) {
    if (!this.state.hasPath) {
      this.setState({
        hasPath: true
      });
    }

    this.path += " ".concat(x, ",").concat(y);
    requestAnimationFrame(this.renderCanvas);
  }

  mouseDown(e) {
    e.preventDefault();
    e.stopImmediatePropagation();
  }

  mouseLeave(e) {
    if (e.buttons === 1) {
      this.appendCoord(e.layerX, e.layerY);
      this.break = true;
    }
  }

  mouseUp() {
    this.break = true;
  }

  mouseEnd(e) {
    e.preventDefault();
    e.stopImmediatePropagation();
    this.break = true;
  }

  appendM() {
    if (!this.path.endsWith(' M')) {
      this.path += ' M';
    }
  }

  mouseMove(e) {
    e.preventDefault();
    e.stopImmediatePropagation();

    if (e.buttons === 1) {
      if (this.break) {
        this.break = false;
        this.appendM();
      }

      const adjusted = this.adjustCoord(e.clientX, e.clientY);

      if (adjusted) {
        this.appendCoord(adjusted.x, adjusted.y);
      } else {
        this.break = true;
      }
    }
  }

  touchStart(e) {
    this.drawing = true;
    this.appendM(); // this prevents to "M"s next to each other if
    // there's a touch start without a touch move

    this.touchMove(e);
    e.preventDefault();
  }

  touchMove(e) {
    if (this.drawing) {
      const touch = e.touches[0];
      const isStylus = touch.touchType === 'stylus';
      const adjusted = this.adjustCoord(touch.clientX + (isStylus ? 0 : -20), touch.clientY + (isStylus ? 0 : -10));

      if (adjusted) {
        this.appendCoord(adjusted.x, adjusted.y);
      } else {
        this.break = true;
      }
    }
  }

  touchEnd() {
    this.break = true;
  }

  renderCanvas() {
    const {
      ctx
    } = this;
    const {
      width,
      height
    } = this.canvas;
    ctx.clearRect(0, 0, width, height);
    const marginFromEdge = width * 0.1;
    const marginFromBottom = 80; // signature line

    ctx.beginPath();
    ctx.moveTo(marginFromEdge, height - marginFromBottom);
    ctx.lineTo(width - marginFromEdge, height - marginFromBottom);
    ctx.strokeStyle = '#ddd';
    ctx.lineWidth = 2;
    ctx.stroke();
    ctx.strokeStyle = 'black';
    ctx.lineWidth = 1;
    ctx.fillStyle = '#000';

    if (this.path) {
      ctx.stroke(new Path2D(this.path));
    }

    if (this.props.name) {
      ctx.font = '18px sans-serif';
      ctx.fillText(this.props.name, marginFromEdge, height - (marginFromBottom - 25));
    }
  }

  shouldComponentUpdate(_, nextState) {
    // only update if haspath has changed
    return this.state.hasPath !== nextState.hasPath;
  }

  cancel() {
    this.clear();
    this.props.close();
  }

  clear(e) {
    if (e) {
      e.preventDefault();
    }

    this.reset();
    this.renderCanvas();
  }

  render() {
    const {
      updateValue,
      showIncompleteWarning,
      confirmButtonText = 'Sign'
    } = this.props;
    const {
      hasPath
    } = this.state;
    return (0, _preact.h)("div", {
      className: "top-0 left-0 fixed bg-light-gray w-100 h-100 z-9999 flex items-center justify-center"
    }, (0, _preact.h)("div", {
      "data-e2e": "signatureBox",
      className: "relative"
    }, showIncompleteWarning && (0, _preact.h)("div", {
      "data-e2e": "incompleteWarning",
      className: "bg-orange flex items-center w-100 pa2 mb2 br2"
    }, (0, _preact.h)(_icon.Icon, {
      icon: "warning",
      className: "mr2"
    }), "The report is incomplete, but as the primary provider you can still sign and finalize it."), (0, _preact.h)("div", {
      className: "relative"
    }, (0, _preact.h)("div", {
      className: "absolute top-0 right-0  flex items-center"
    }, (0, _preact.h)(_links.Link, {
      href: "#",
      onClick: this.clear,
      "data-e2e": "clearSignatureButton",
      className: 'underline gray ' + (hasPath ? 'o-100' : 'o-0')
    }, "Clear Signature"), (0, _preact.h)("button", {
      onClick: this.cancel,
      className: "pointer button-reset bn bg-transparent db pa3 flex items-center dark-blue"
    }, (0, _preact.h)(_icon.Icon, {
      icon: "close"
    }))), (0, _preact.h)("canvas", {
      "data-e2e": "signaturePad",
      className: "bg-white ba br2 b--gray",
      width: Math.min(window.innerWidth - 10, 660),
      height: height,
      onMouseMove: this.mouseMove,
      onMouseDown: this.mouseDown,
      onMouseCancel: this.mouseCancel,
      onMouseLeave: this.mouseLeave
    })), (0, _preact.h)("div", {
      className: "flex items-center justify-end mt2"
    }, (0, _preact.h)(_button.Button, {
      primary: true,
      large: true,
      disabled: !hasPath,
      className: hasPath ? 'o-100' : 'o-0',
      "data-e2e": "signatureSaveButton",
      onClick: () => {
        updateValue(this.path.trim());
      },
      type: "button"
    }, confirmButtonText))));
  }

}

exports.SignatureInput = SignatureInput;